package cz.fi.muni.pa165;

import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cloud.gateway.route.RouteLocator;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.web.server.ServerHttpSecurity;
import org.springframework.security.oauth2.client.oidc.web.server.logout.OidcClientInitiatedServerLogoutSuccessHandler;
import org.springframework.security.oauth2.client.registration.ReactiveClientRegistrationRepository;
import org.springframework.security.web.server.SecurityWebFilterChain;
import org.springframework.security.web.server.header.XFrameOptionsServerHttpHeadersWriter;

@SpringBootApplication
@OpenAPIDefinition(servers = {
		@Server(url = "/", description = "Default Server URL")
}, info = @Info(title = "API Gateway", version = "1.0", description = "Documentation API Gateway v1.0"))
public class BandApiGwApplication {

	public static void main(String[] args) {
		SpringApplication.run(BandApiGwApplication.class, args);
	}

	@Bean
	public RouteLocator routeLocator(RouteLocatorBuilder builder) {
		return builder
				.routes()
				.route(r -> r.path("/api/v1/users/v3/api-docs").and().method(HttpMethod.GET)
						.uri("http://pa165-user-service:9091/api/v1/users/v3/api-docs"))
				.route(r -> r.path("/api/v1/bands/v3/api-docs").and().method(HttpMethod.GET)
						.uri("http://pa165-band-management-service:9092/api/v1/bands/v3/api-docs"))
				.route(r -> r.path("/api/v1/team-members/v3/api-docs").and().method(HttpMethod.GET)
						.uri("http://pa165-team-member-service:9093/api/v1/team-members/v3/api-docs"))
				.route(r -> r.path("/api/v1/band-manager/v3/api-docs").and().method(HttpMethod.GET)
						.uri("http://pa165-band-manager-service:9094/api/v1/band-manager/v3/api-docs"))
    .build();
	}

	@Bean
	public SecurityWebFilterChain springSecurityFilterChain(ServerHttpSecurity http,
															ReactiveClientRegistrationRepository clientRegistrationRepository) {
		// Authenticate through configured OpenID Provider
		http.oauth2Login();

		// Also logout at the OpenID Connect provider
		http.logout(logout -> logout.logoutSuccessHandler(
				new OidcClientInitiatedServerLogoutSuccessHandler(clientRegistrationRepository)));

		// Require authentication for all requests
		http.authorizeExchange()
				.pathMatchers("/actuator/health").permitAll()
				.pathMatchers("/actuator/info").permitAll()
				.pathMatchers("/actuator/metrics").permitAll()
				.pathMatchers("/actuator/prometheus").permitAll()
				.anyExchange().authenticated();

		// Allow showing /home within a frame
		http.headers().frameOptions().mode(XFrameOptionsServerHttpHeadersWriter.Mode.SAMEORIGIN);

		// Disable CSRF in the gateway to prevent conflicts with proxied service CSRF
		http.csrf().disable();
		return http.build();
	}
}
